function [] = printadmresults(adm_in, adm_out);
% printadmresults.m
% Prints the inputs and outputs of the BSM2 ADM1 model
% adm_in is the input in ADM1 terminology + extra states, 33 variables
% adm_in[1] : Ssu = monosacharides (kg COD/m3)
% adm_in[2] : Saa = amino acids (kg COD/m3)
% adm_in[3] : Sfa = long chain fatty acids (LCFA) (kg COD/m3)
% adm_in[4] : Sva = total valerate (kg COD/m3)
% adm_in[5] : Sbu = total butyrate (kg COD/m3)
% adm_in[6] : Spro = total propionate (kg COD/m3)
% adm_in[7] : Sac = total acetate (kg COD/m3)
% adm_in[8] : Sh2 = hydrogen gas (kg COD/m3)
% adm_in[9] : Sch4 = methane gas (kg COD/m3)
% adm_in[10] : Sic = inorganic carbon (kmole C/m3)
% adm_in[11] : Sin = inorganic nitrogen (kmole N/m3)
% adm_in[12] : Si = soluble inerts (kg COD/m3)
% adm_in[13] : Xc = composites (kg COD/m3)
% adm_in[14] : Xch = carbohydrates (kg COD/m3)
% adm_in[15] : Xpr = proteins (kg COD/m3)
% adm_in[16] : Xli = lipids (kg COD/m3)
% adm_in[17] : Xsu = sugar degraders (kg COD/m3)
% adm_in[18] : Xaa = amino acid degraders (kg COD/m3)
% adm_in[19] : Xfa = LCFA degraders (kg COD/m3)
% adm_in[20] : Xc4 = valerate and butyrate degraders (kg COD/m3)
% adm_in[21] : Xpro = propionate degraders (kg COD/m3)
% adm_in[22] : Xac = acetate degraders (kg COD/m3)
% adm_in[23] : Xh2 = hydrogen degraders (kg COD/m3)
% adm_in[24] : Xi = particulate inerts (kg COD/m3)
% adm_in[25] : scat+ = cations (metallic ions, strong base) (kmole/m3)
% adm_in[26] : san- = anions (metallic ions, strong acid) (kmole/m3)
% adm_in[27] : flow rate (m3/d)
% adm_in[28] : temperature (deg C)
% adm_in[29:33] : dummy states for future use
%
% adm_out is the output in ADM1 terminology + extra states, 50 variables
% adm_out[1] : Ssu = monosacharides (kg COD/m3)
% adm_out[2] : Saa = amino acids (kg COD/m3)
% adm_out[3] : Sfa = long chain fatty acids (LCFA) (kg COD/m3)
% adm_out[4] : Sva = total valerate (kg COD/m3)
% adm_out[5] : Sbu = total butyrate (kg COD/m3)
% adm_out[6] : Spro = total propionate (kg COD/m3)
% adm_out[7] : Sac = total acetate (kg COD/m3)
% adm_out[8] : Sh2 = hydrogen gas (kg COD/m3)
% adm_out[9] : Sch4 = methane gas (kg COD/m3)
% adm_out[10] : Sic = inorganic carbon (kmole C/m3)
% adm_out[11] : Sout = inorganic nitrogen (kmole N/m3)
% adm_out[12] : Si = soluble inerts (kg COD/m3)
% adm_out[13] : Xc = composites (kg COD/m3)
% adm_out[14] : Xch = carbohydrates (kg COD/m3)
% adm_out[15] : Xpr = proteins (kg COD/m3)
% adm_out[16] : Xli = lipids (kg COD/m3)
% adm_out[17] : Xsu = sugar degraders (kg COD/m3)
% adm_out[18] : Xaa = amino acid degraders (kg COD/m3)
% adm_out[19] : Xfa = LCFA degraders (kg COD/m3)
% adm_out[20] : Xc4 = valerate and butyrate degraders (kg COD/m3)
% adm_out[21] : Xpro = propionate degraders (kg COD/m3)
% adm_out[22] : Xac = acetate degraders (kg COD/m3)
% adm_out[23] : Xh2 = hydrogen degraders (kg COD/m3)
% adm_out[24] : Xi = particulate inerts (kg COD/m3)
% adm_out[25] : scat+ = cations (metallic ions, strong base) (kmole/m3)
% adm_out[26] : san- = anions (metallic ions, strong acid) (kmole/m3)
% adm_out[27] : flow rate (m3/d)
% adm_out[28] : temperature (deg C)
% adm_out[29:33] : dummy states for future use
% adm_out[34] : pH = pH within AD system
% adm_out[35] : S_H+ = protons (kmole/m3)
% adm_out[36] : Sva- = valerate (kg COD/m3)
% adm_out[37] : Sbu- = butyrate (kg COD/m3)
% adm_out[38] : Spro- = propionate (kg COD/m3)
% adm_out[39] : Sac- = acetate (kg COD/m3)
% adm_out[40] : Shco3- = bicarbonate (kmole C/m3)
% adm_out[41] : Sco2 = carbon dioxide (kmole C/m3)
% adm_out[42] : Snh3 = ammonia (kmole N/m3)
% adm_out[43] : Snh4+ = ammonium (kmole N/m3)
% adm_out[44] : Sgas,h2 = hydrogen concentration in gas phase (kg COD/m3)
% adm_out[45] : Sgas,ch4 = methane concentration in gas phase (kg COD/m3)
% adm_out[46] : Sgas,co2 = carbon dioxide concentration in gas phase (kmole C/m3)
% adm_out[47] : pgas,h2 = partial pressure of hydrogen gas (bar)
% adm_out[48] : pgas,ch4 = partial pressure of methane gas (bar)
% adm_out[49] : pgas,co2 = partial pressure of carbon dioxide gas (bar)
% adm_out[50] : pgas,total = total head space pressure (H2+CO2+CH4+H2O) (bar)
% adm_out[51] : qgas = gas flow rate normalised to atmospheric pressure (m3/d)


disp(' ')
disp('ADM1 influent')
disp('*************')
disp(['adm_in[1] : Ssu = monosacharides (kg COD/m3) = ', num2str(adm_in(end,1))]);
disp(['adm_in[2] : Saa = amino acids (kg COD/m3) = ', num2str(adm_in(end,2))]);
disp(['adm_in[3] : Sfa = long chain fatty acids (LCFA) (kg COD/m3) = ', num2str(adm_in(end,3))]);
disp(['adm_in[4] : Sva = total valerate (kg COD/m3) = ', num2str(adm_in(end,4))]);
disp(['adm_in[5] : Sbu = total butyrate (kg COD/m3) = ', num2str(adm_in(end,5))]);
disp(['adm_in[6] : Spro = total propionate (kg COD/m3) = ', num2str(adm_in(end,6))]);
disp(['adm_in[7] : Sac = total acetate (kg COD/m3) = ', num2str(adm_in(end,7))]);
disp(['adm_in[8] : Sh2 = hydrogen gas (kg COD/m3) = ', num2str(adm_in(end,8))]);
disp(['adm_in[9] : Sch4 = methane gas (kg COD/m3) = ', num2str(adm_in(end,9))]);
disp(['adm_in[10] : Sic = inorganic carbon (kmole C/m3) = ', num2str(adm_in(end,10))]);
disp(['adm_in[11] : Sin = inorganic nitrogen (kmole N/m3) = ', num2str(adm_in(end,11))]);
disp(['adm_in[12] : Si = soluble inerts (kg COD/m3) = ', num2str(adm_in(end,12))]);
disp(['adm_in[13] : Xc = composites (kg COD/m3) = ', num2str(adm_in(end,13))]);
disp(['adm_in[14] : Xch = carbohydrates (kg COD/m3) = ', num2str(adm_in(end,14))]);
disp(['adm_in[15] : Xpr = proteins (kg COD/m3) = ', num2str(adm_in(end,15))]);
disp(['adm_in[16] : Xli = lipids (kg COD/m3) = ', num2str(adm_in(end,16))]);
disp(['adm_in[17] : Xsu = sugar degraders (kg COD/m3) = ', num2str(adm_in(end,17))]);
disp(['adm_in[18] : Xaa = amino acid degraders (kg COD/m3) = ', num2str(adm_in(end,18))]);
disp(['adm_in[19] : Xfa = LCFA degraders (kg COD/m3) = ', num2str(adm_in(end,19))]);
disp(['adm_in[20] : Xc4 = valerate and butyrate degraders (kg COD/m3) = ', num2str(adm_in(end,20))]);
disp(['adm_in[21] : Xpro = propionate degraders (kg COD/m3) = ', num2str(adm_in(end,21))]);
disp(['adm_in[22] : Xac = acetate degraders (kg COD/m3) = ', num2str(adm_in(end,22))]);
disp(['adm_in[23] : Xh2 = hydrogen degraders (kg COD/m3) = ', num2str(adm_in(end,23))]);
disp(['adm_in[24] : Xi = particulate inerts (kg COD/m3) = ', num2str(adm_in(end,24))]);
disp(['adm_in[25] : Scat+ = cations (base) (kmole/m3) = ', num2str(adm_in(end,25))]);
disp(['adm_in[26] : San- = anions (acid) (kmole/m3) = ', num2str(adm_in(end,26))]);
disp(['adm_in[27] : Flow rate (m3/d) = ', num2str(adm_in(end,27))]);
disp(['adm_in[28] : Temperature (degC) = ', num2str(adm_in(end,28))]);
disp(['adm_in[29] : Dummy soluble state no 1 (g/m3) = ', num2str(adm_in(end,29))]);
disp(['adm_in[30] : Dummy soluble state no 2 (g/m3) = ', num2str(adm_in(end,30))]);
disp(['adm_in[31] : Dummy soluble state no 3 (g/m3) = ', num2str(adm_in(end,31))]);
disp(['adm_in[32] : Dummy particulate state no 1 (g/m3) = ', num2str(adm_in(end,32))]);
disp(['adm_in[33] : Dummy particulate state no 2 (g/m3) = ', num2str(adm_in(end,33))]);

disp(' ')
disp('ADM1 effluent')
disp('*************')
disp(['adm_out[1] : Ssu = monosacharides (kg COD/m3) = ', num2str(adm_out(end,1))]);
disp(['adm_out[2] : Saa = amino acids (kg COD/m3) = ', num2str(adm_out(end,2))]);
disp(['adm_out[3] : Sfa = long chain fatty acids (LCFA) (kg COD/m3) = ', num2str(adm_out(end,3))]);
disp(['adm_out[4] : Sva = total valerate (kg COD/m3) = ', num2str(adm_out(end,4))]);
disp(['adm_out[5] : Sbu = total butyrate (kg COD/m3) = ', num2str(adm_out(end,5))]);
disp(['adm_out[6] : Spro = total propionate (kg COD/m3) = ', num2str(adm_out(end,6))]);
disp(['adm_out[7] : Sac = total acetate (kg COD/m3) = ', num2str(adm_out(end,7))]);
disp(['adm_out[8] : Sh2 = hydrogen gas (kg COD/m3) = ', num2str(adm_out(end,8))]);
disp(['adm_out[9] : Sch4 = methane gas (kg COD/m3) = ', num2str(adm_out(end,9))]);
disp(['adm_out[10] : Sic = inorganic carbon (kmole C/m3) = ', num2str(adm_out(end,10))]);
disp(['adm_out[11] : Sin = inorganic nitrogen (kmole N/m3) = ', num2str(adm_out(end,11))]);
disp(['adm_out[12] : Si = soluble inerts (kg COD/m3) = ', num2str(adm_out(end,12))]);
disp(['adm_out[13] : Xc = composites (kg COD/m3) = ', num2str(adm_out(end,13))]);
disp(['adm_out[14] : Xch = carbohydrates (kg COD/m3) = ', num2str(adm_out(end,14))]);
disp(['adm_out[15] : Xpr = proteins (kg COD/m3) = ', num2str(adm_out(end,15))]);
disp(['adm_out[16] : Xli = lipids (kg COD/m3) = ', num2str(adm_out(end,16))]);
disp(['adm_out[17] : Xsu = sugar degraders (kg COD/m3) = ', num2str(adm_out(end,17))]);
disp(['adm_out[18] : Xaa = amino acid degraders (kg COD/m3) = ', num2str(adm_out(end,18))]);
disp(['adm_out[19] : Xfa = LCFA degraders (kg COD/m3) = ', num2str(adm_out(end,19))]);
disp(['adm_out[20] : Xc4 = valerate and butyrate degraders (kg COD/m3) = ', num2str(adm_out(end,20))]);
disp(['adm_out[21] : Xpro = propionate degraders (kg COD/m3) = ', num2str(adm_out(end,21))]);
disp(['adm_out[22] : Xac = acetate degraders (kg COD/m3) = ', num2str(adm_out(end,22))]);
disp(['adm_out[23] : Xh2 = hydrogen degraders (kg COD/m3) = ', num2str(adm_out(end,23))]);
disp(['adm_out[24] : Xi = particulate inerts (kg COD/m3) = ', num2str(adm_out(end,24))]);
disp(['adm_out[25] : Scat+ = cations (base) (kmole/m3) = ', num2str(adm_out(end,25))]);
disp(['adm_out[26] : San- = anions (acid) (kmole/m3) = ', num2str(adm_out(end,26))]);
disp(['adm_out[27] : Flow rate (m3/d) = ', num2str(adm_out(end,27))]);
disp(['adm_out[28] : Temperature (degC) = ', num2str(adm_out(end,28))]);
disp(['adm_out[29] : Dummy soluble state no 1 (g/m3) = ', num2str(adm_out(end,29))]);
disp(['adm_out[30] : Dummy soluble state no 2 (g/m3) = ', num2str(adm_out(end,30))]);
disp(['adm_out[31] : Dummy soluble state no 3 (g/m3) = ', num2str(adm_out(end,31))]);
disp(['adm_out[32] : Dummy particulate state no 1 (g/m3) = ', num2str(adm_out(end,32))]);
disp(['adm_out[33] : Dummy particulate state no 2 (g/m3) = ', num2str(adm_out(end,33))]);
disp(['adm_out[34] : pH = pH within AD system = ', num2str(adm_out(end,34))]);
disp(['adm_out[35] : S_H+ = protons (kmole/m3) = ', num2str(adm_out(end,35))]);
disp(['adm_out[36] : Sva- = valerate (kg COD/m3) = ', num2str(adm_out(end,36))]);
disp(['adm_out[37] : Sbu- = butyrate (kg COD/m3) = ', num2str(adm_out(end,37))]);
disp(['adm_out[38] : Spro- = propionate (kg COD/m3) = ', num2str(adm_out(end,38))]);
disp(['adm_out[39] : Sac- = acetate (kg COD/m3) = ', num2str(adm_out(end,39))]);
disp(['adm_out[40] : Shco3- = bicarbonate (kmole C/m3) = ', num2str(adm_out(end,40))]);
disp(['adm_out[41] : Sco2 = carbon dioxide (kmole C/m3) = ', num2str(adm_out(end,41))]);
disp(['adm_out[42] : Snh3 = ammonia (kmole N/m3) = ', num2str(adm_out(end,42))]);
disp(['adm_out[43] : Snh4+ = ammonium (kmole N/m3) = ', num2str(adm_out(end,43))]);
disp(['adm_out[44] : Sgas,h2 = hydrogen concentration in gas phase (kg COD/m3) = ', num2str(adm_out(end,44))]);
disp(['adm_out[45] : Sgas,ch4 = methane concentration in gas phase (kg COD/m3) = ', num2str(adm_out(end,45))]);
disp(['adm_out[46] : Sgas,co2 = carbon dioxide concentration in gas phase (kmole C/m3) = ', num2str(adm_out(end,46))]);
disp(['adm_out[47] : pgas,h2 = partial pressure of hydrogen gas (bar, true value i.e. not normalized) = ', num2str(adm_out(end,47))]);
disp(['adm_out[48] : pgas,ch4 = partial pressure of methane gas (bar, true value i.e. not normalized) = ', num2str(adm_out(end,48))]);
disp(['adm_out[49] : pgas,co2 = partial pressure of carbon dioxide gas (bar, true value, i.e. not normalized) = ', num2str(adm_out(end,49))]);
disp(['adm_out[50] : pgas,total = total head space pressure of H2+CO2+CH4+H2O (bar, true value, i.e. not normalized) = ', num2str(adm_out(end,50))]);
disp(['adm_out[51] : qgas = gas flow rate normalized to atmospheric pressure (m3/d) = ', num2str(adm_out(end,51))]);
disp(' ' )